#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/xmlerror.h>
#include <libxml/uri.h>
#include <libxslt/xslt.h>
#include <libxslt/xsltInternals.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>

const char XML_DETAIL[] = "http://dict.youdao.com/search?keyfrom=\
deskdict.main&xmlDetail=true&doctype=xml&\
xmlVersion=6.1&dogVersion=1.0&client=deskdict&q=";
const char XSL_DETAIL[] = "/opt/youdaodict/detail/result.xsl";
const char XML_SIMPLE[] = "http://dict.youdao.com/search?\
ue=utf8&doctype=xml&q=";
const char XSL_SIMPLE[] = "/opt/youdaodict/simple/result.xsl";
const char CONFIG_PATH[] = "/opt/youdaodict/config";
xmlChar *queryWord = 0;
int isDetail = 1;
int fromConfig = 0;

int get_config(char **xml,char **xsl);
int read_config(char **xml,char **xsl);
int check_argument(int argc,char *argv[]);
void do_xslt(const char *xmlAPI,const char *xslAPI);
void print_usage(const char *name);

int main(int argc,char *argv[])
{
	if(check_argument(argc,argv))
	{
		print_usage(argv[0]);
		return 1;
	}

	char *xml = NULL,*xsl = NULL;
	if(get_config(&xml,&xsl))
	{
		print_usage(argv[0]);
		xmlFree(queryWord);
		return 1;
	}

	do_xslt(xml,xsl);

	free(xml);
	free(xsl);
	xmlFree(queryWord);
	return 0;
}

int get_config(char **xml,char **xsl)
{
	*xml = *xsl = NULL;
	if(fromConfig)
		return read_config(xml,xsl);

	const char *XML;
	const char *XSL;
	if(isDetail)
	{
		XML = XML_DETAIL;
		XSL = XSL_DETAIL;
	}
	else
	{
		XML = XML_SIMPLE;
		XSL = XSL_SIMPLE;
	}

	/*	if fromConfig == 0, then queryWord != 0	*/
	*xsl = (char*) malloc (strlen(XSL) + 1);
	sprintf(*xsl,"%s",XSL);
	*xml = (char*) malloc (strlen(XML) + strlen(queryWord) + 1);
	sprintf(*xml,"%s%s",XML,queryWord);

	return 0;
}

int read_config(char **xml,char **xsl)
{
	char xmlKey[128],xslKey[128];
	if(isDetail)
	{
		sprintf(xmlKey,"xmldetail");
		sprintf(xslKey,"xsldetail");
	}
	else
	{
		sprintf(xmlKey,"xmlsimple");
		sprintf(xslKey,"xslsimple");
	}

	xmlDocPtr doc = 0;
	xmlNodePtr root = 0,cur = 0;
	doc = xmlReadFile(CONFIG_PATH,"utf-8",0);
	if(!doc)
	{
		fprintf(stdout,"open /opt/youdaodict/config failed\n");
		return 1;
	}

	root = xmlDocGetRootElement(doc);
	cur = root->children;
	while(cur)
	{
		if(strcmp(cur->name,xmlKey) == 0)
		{
			*xml = (char*) malloc (strlen(xmlNodeGetContent(cur)) + 
				(queryWord ? strlen(queryWord) : 0) + 1);
			sprintf(*xml,"%s",xmlNodeGetContent(cur));
		}
		if(strcmp(cur->name,xslKey) == 0)
		{
			*xsl = (char*) malloc (strlen(xmlNodeGetContent(cur)) + 1);
			sprintf(*xsl,"%s",xmlNodeGetContent(cur));
		}
		cur = cur->next;
	}

	if(!*xml && !*xsl)
	{
		fprintf(stdout,"read /opt/youdaodict/config error\n");
		xmlFreeNode(root);
		xmlFreeDoc(doc);
		return 1;
	}
	if(queryWord)
		strcat(*xml,queryWord);
	xmlFreeDoc(doc);
	return 0;
}

int check_argument(int argc,char *argv[])
{
	int i;
	for(i = 1;i < argc; ++i)
	{
		if(strcmp(argv[i],"-simple") == 0)
			isDetail = 0;
		else if(strcmp(argv[i],"-detail") == 0)
			isDetail = 1;
		else if(strcmp(argv[i],"-config") == 0)
			fromConfig = 1;
		else if(strcmp(argv[i],"-keyword") == 0)
		{
			++i;
			queryWord = xmlPathToURI((const xmlChar*)argv[i]);
		}
		else
		{
			xmlFree(queryWord);
			fprintf(stdout,"unknow arguments\n");
			return 1;
		}
	}

	if(queryWord == 0 && fromConfig == 0)
	{
		fprintf(stdout,"use -keyword or -config\n");
		return 1;
	}

	return 0;
}

void do_xslt(const char *xmlAPI,const char *xslAPI)
{
	xmlDocPtr doc,res;
	xsltStylesheetPtr cur;
	xmlErrorPtr errmsg = NULL;
	doc = res = NULL;
	cur = NULL;
	int success = 0;
	xmlSubstituteEntitiesDefault(1);
	xmlLoadExtDtdDefaultValue = 1;

	doc	= xmlReadFile((const xmlChar*)xmlAPI,"utf-8",XML_PARSE_NOERROR);
	if(!doc)	goto end;

	cur = xsltParseStylesheetFile((const xmlChar *)xslAPI);
	if(!cur)	goto end;

	res = xsltApplyStylesheet(cur, doc, NULL);
	if(!res)	goto end;

	if(xsltSaveResultToFile(stdout, res, cur) == -1)	
		goto end;
	
	success = 1;
end:
	if(!success)
	{
		errmsg = xmlGetLastError();
		fprintf(stdout,"<html><body><p>Error Occur!<br />\
%s<br />\
check your configuration file and the network</p></body></html>",
		errmsg ? errmsg->message : "");
		xmlResetLastError();
	}
	xsltFreeStylesheet(cur);
	xmlFreeDoc(res);
	xmlFreeDoc(doc);
	xsltCleanupGlobals();
	xmlCleanupParser();
	return;
}

void print_usage(const char *name)
{
	printf("usage:\n\t%s [-config] [-simple|-detail] [-keyword word]\n",name);
}
